package org.jboss.jbossws.jbws3690;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.ConnectException;
import java.net.URL;

import org.apache.commons.io.IOUtils;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.RunAsClient;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.StringAsset;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(Arquillian.class)
@RunAsClient
public class JBWS3690IT {


  private static final String name = "JBWS3690";

  @Deployment
  static WebArchive creatDeployment() {
     WebArchive archive = ShrinkWrap.create(WebArchive.class, name + ".war")
           .setManifest(new StringAsset("Manifest-Version: 1.0\n"
                 + "Dependencies: org.jboss.ws.cxf.jbossws-cxf-client\n"))
           .addClass(WebService.class)
           .addClass(MyException.class)
           .addClass(NoArgObj.class)
           .addClass(NoArgObjAdapter.class)
           .addClass(Cl1ToCl2Adapter.class)
           .addClass(MyClass1.class)
           .addClass(MyClass2.class);
     //archive.as(ZipExporter.class).exportTo(new File("/tmp", archive.getName()), true);
     return archive;
  }

  @Test
  public void testWsdlContainsMapping() throws Exception {
    String url = "http://localhost:8080/" + name + "/" + WebService.class.getSimpleName() + "?wsdl";
    String wsdl = readFromUrlToString(new URL(url), "utf-8");
    String expectedMapping = "<xs:element name=\"obj1\" nillable=\"true\" type=\"tns:myClass2\"/>";
    assertTrue("Not found expected mapping " + expectedMapping + " in " + wsdl, wsdl.contains(expectedMapping));
  }

  public static String readFromUrlToString(URL url, String encoding) throws UnsupportedEncodingException, IOException
  {
     InputStream stream = null;
     try {
        stream = url.openStream();
     } catch (ConnectException e) {
        throw new IllegalStateException("Can not read from " + url.toString(), e);
     }
     InputStreamReader inputStream = null;
     try {
        inputStream = new InputStreamReader(stream, encoding);
        return IOUtils.toString(inputStream);
     } finally {
        IOUtils.closeQuietly(inputStream);
     }
  }
}
